from napari.viewer import Viewer
from tifffile import imread
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from pathlib import Path
from  napari import layers
class OneatSimpleVisualization:
    
    def __init__(self, viewer: Viewer,
                 ax,
                 figure: plt):
        
        self.viewer = viewer 
        self.ax = ax
        self.figure = figure
        
        
    def show_csv(self, csvname, event_threshold):
                   
            self.ax.cla()       
            for layer in list(self.viewer.layers):
               if isinstance(layer, layers.Image):
                  self.image = layer.data
        
            self.totaltime = self.image.shape[0]
                
            event_locations_dict = {}
            event_locations_size_dict = {}
            event_locations = []     
            size_locations = [] 
            score_locations = []
            confidence_locations = [] 
            current_list = []
            detections = pd.read_csv(csvname, delimiter=',')
            nrows = len(detections.columns)
            for index, row in detections.iterrows():
                tcenter = int(row[0])
                zcenter = row[1]
                ycenter = row[2]
                xcenter = row[3]
                if nrows > 4:
                    score = row[4]
                    size = row[5]
                    confidence = row[6]
                else:
                    score = 1.0
                    size = 10
                    confidence = 1.0
                if score > event_threshold:
                    event_locations.append(
                        [
                            int(tcenter),
                            int(zcenter),
                            int(ycenter),
                            int(xcenter),
                        ]
                    )

                    if int(tcenter) in event_locations_dict.keys():
                        current_list = event_locations_dict[int(tcenter)]
                        current_list.append(
                            [int(zcenter), int(ycenter), int(xcenter)]
                        )
                        event_locations_dict[int(tcenter)] = current_list
                        event_locations_size_dict[
                            (
                                int(tcenter),
                                int(zcenter),
                                int(ycenter),
                                int(xcenter),
                            )
                        ] = [size, score, confidence]
                    else:
                        current_list = []
                        current_list.append(
                            [int(zcenter), int(ycenter), int(xcenter)]
                        )
                        event_locations_dict[int(tcenter)] = current_list
                        event_locations_size_dict[
                            int(tcenter),
                            int(zcenter),
                            int(ycenter),
                            int(xcenter),
                        ] = [size, score, confidence]

                    size_locations.append(size)
                    score_locations.append(score)
                    confidence_locations.append(confidence)
            point_properties = {
                "score": np.array(score_locations),
                "confidence": np.array(confidence_locations),
                "size": np.array(size_locations),
            }

            name_remove = ("Detections", "Location Map")
            for layer in list(self.viewer.layers):

                if any(name in layer.name for name in name_remove):
                    self.viewer.layers.remove(layer)
            if len(score_locations) > 0:
                self.viewer.add_points(
                    event_locations,
                    properties=point_properties,
                    symbol="square",
                    blending="translucent_no_depth",
                    name="Detections",
                    face_color=[0] * 4,
                    edge_color="red",
                )
                
            timelist = []
            countlist = []    
            for tcenter in range(self.totaltime):
                
                timelist.append(tcenter)
                if int(tcenter) in event_locations_dict:
                  countlist.append(len(event_locations_dict[int(tcenter)]))
                else:
                    countlist.append(0)  
                
             
            self.ax.plot(timelist, countlist, "-g")
            self.ax.set_title("Events")
            self.ax.set_xlabel("Time")
            self.ax.set_ylabel("Counts")
            self.figure.canvas.draw()
            self.figure.canvas.flush_events()

            self.figure.savefig(
                 str(Path(csvname).parent.as_posix())
                + str('visualplot')
                + str('.png'),
                  dpi=300
            )         
                        


import csv
import os

import numpy as np
from scipy.spatial import KDTree

try:
    from pathlib import Path

    Path().expanduser()
except (ImportError, AttributeError):
    from pathlib2 import Path

try:
    import tempfile

    tempfile.TemporaryDirectory

except (ImportError, AttributeError):
    from backports import tempfile


def compute_nearest(box, boxes, secondboxes, mindist=20):

    xbox = box[4]
    ybox = box[5]

    pts = []
    if boxes is not None:
        for i in range(0, boxes.shape[0]):
            xboxes = boxes[i, 4]
            yboxes = boxes[i, 5]
            center = (xboxes, yboxes)
            pts.append(center)
    secondpts = []
    if secondboxes is not None:
        for i in range(0, secondboxes.shape[0]):
            xboxes = secondboxes[i, 4]
            yboxes = secondboxes[i, 5]
            center = (xboxes, yboxes)
            secondpts.append(center)
    indexA = None
    indexB = None
    if len(pts) > 0:
        pts = np.array(pts)
        Target = KDTree(pts)
        if Target is not None:
            idx = Target.query_ball_point([xbox, ybox], r=mindist)
            if idx is not None:
                # Return the list of nearest points
                Nearest = pts[idx]
                SortNearest = np.sort(Nearest[::-1])
                if len(SortNearest) > 1:
                    NearestXY = SortNearest[0]
                    if boxes is not None:
                        for i in range(0, boxes.shape[0]):
                            if (
                                boxes[i, 4] == NearestXY[0]
                                and boxes[i, 5] == NearestXY[1]
                            ):
                                indexA = i
        if len(secondpts) > 0:
            secondpts = np.array(secondpts)
            secondTarget = KDTree(secondpts)
            # Get id array of all nearest points

            if secondTarget is not None:
                secondidx = secondTarget.query_ball_point(
                    [xbox, ybox], r=mindist
                )

            if secondidx is not None:
                secondNearest = secondpts[secondidx]
                # Sort the list of nearest points

                SortsecondNearest = np.sort(secondNearest[::-1])

                if len(SortsecondNearest > 1):
                    secondNearestXY = SortsecondNearest[0]

                    if secondboxes is not None:
                        for i in range(0, secondboxes.shape[0]):
                            if (
                                secondboxes[i, 4] == secondNearestXY[0]
                                and secondboxes[i, 5] == secondNearestXY[1]
                            ):
                                indexB = i

    return indexA, indexB


def drawimage(eventlist, basedirResults, fname, Label):
    if eventlist is not None:

        LocationXlist = []
        LocationYlist = []
        Timelist = []

        for i in range(0, len(eventlist)):
            location, time, Name = eventlist[i]
            (x, y) = location
            returntime = int(time)

            LocationXlist.append(location[0])
            LocationYlist.append(location[1])
            Timelist.append(returntime)

        Event_Count = np.column_stack([LocationXlist, LocationYlist, Timelist])
        Event_data = []

        for line in Event_Count:
            Event_data.append(line)
            writer = csv.writer(
                open(
                    basedirResults
                    + "/"
                    + str(Label)
                    + "LocationEventCounts"
                    + (os.path.splitext(os.path.basename(fname))[0])
                    + ".csv",
                    "a",
                )
            )
            writer.writerows(Event_data)
            Event_data = []


def StaticEvents(
    Image,
    timelist,
    TimeFrames,
    boxes,
    originalsize,
    threshold,
    timepoint,
    basedirResults,
    axes,
    fname,
    AppendName,
):

    originalsizeX, originalsizeY = originalsize

    EventList, EventBoxes = NMSSpace(
        boxes, originalsizeX, originalsizeY, threshold
    )

    return [], EventList


def DynamicEvents(
    Image,
    timelist,
    TimeFrames,
    boxes,
    originalsize,
    threshold,
    timepoint,
    basedirResults,
    axes,
    fname,
    AppendName,
):

    originalsizeX, originalsizeY = originalsize
    EventList, boxes = NMSSpace(boxes, originalsizeX, originalsizeY, threshold)
    if len(EventList) > 0:
        boxes = boxes.tolist()
    EventList = []
    if timepoint % (TimeFrames + 2) == 0 and timepoint != 0:

        EventList, EventBoxes = NMSSpace(
            boxes, originalsizeX, originalsizeY, threshold
        )

        boxes = []

    return boxes, EventList


def NMSSpace(boxes, originalsizeX, originalsizeY, threshold):

    if boxes is not None:
        if len(boxes) == 0:
            return [], []
        else:
            boxes = np.array(boxes, dtype=float)
            assert boxes.shape[0] > 0
            if boxes.dtype.kind != "f":
                boxes = boxes.astype(np.float32)

            score = -boxes[:, 6]
            idxs = np.argsort(score)

            pick = []
            while len(idxs) > 0:
                i = idxs[0]
                pick.append(i)

                distance = compute_dist(boxes[i], boxes[idxs[1:]])

                remove_idxs = np.where(distance < threshold)[0] + 1
                idxs = np.delete(idxs, remove_idxs)
                idxs = np.delete(idxs, 0)

            centerlist = []

            for i in range(len(boxes[pick, 0])):

                center = ((boxes[pick, 4][i]), (boxes[pick, 5][i]))
                centerlist.append(
                    [center, boxes[pick, 7][i], boxes[pick, 8][i]]
                )
            if boxes[pick, 8][0] == 1:
                Name = "Apoptosis"
            if boxes[pick, 8][0] == 2:
                Name = "Division"
            if boxes[pick, 8][0] == 3:
                Name = "MacroKitty"
            if boxes[pick, 8][0] == 4:
                Name = "Non-Mature"
            if boxes[pick, 8][0] == 5:
                Name = "Mature"
            print("Number of events:", len(centerlist), Name)

            return centerlist, boxes[pick]


def compute_dist(box, boxes):
    # Calculate intersection areas

    Xtarget = boxes[:, 4]
    Ytarget = boxes[:, 5]
    Ttarget = boxes[:, 7]

    Xsource = box[4]
    Ysource = box[5]
    Tsource = box[7]

    # If seperated in time the distance is made lower to avoid multi counting of events

    distance = (
        (Xtarget - Xsource) * (Xtarget - Xsource)
        + (Ytarget - Ysource) * (Ytarget - Ysource)
        - (Ttarget - Tsource) * (Ttarget - Tsource)
    )

    distance[distance < 0] = 0

    return np.sqrt(distance)



import pytest
from .utils import random_image_4d, create_train_val_data, root_dir
from oneat.NEATModels import NEATDenseVollNet, config

@pytest.mark.parametrize('shape', [(8,64,64,3), (8,32,32,3)])
@pytest.mark.parametrize('depth', [{'depth_0' :1, 'depth_2' :1, 'depth_3' :1}])
@pytest.mark.parametrize('stage_number', [3])
def test_densent_input(shape, depth, stage_number):
    
    img = random_image_4d(shape)
    catconfig = {'Normal': 0, 'Division': 1}
    tmpdir = root_dir()
    cordconfig = {"x": 0, "y": 1, "z": 2, "t": 3, "h": 4, "w": 5, "d": 6, "c": 7}
    params = config.volume_config(key_categories = catconfig, key_cord = cordconfig, epochs = 0, depth = depth, stage_number = stage_number)
    oneat_model = NEATDenseVollNet(params, str(tmpdir), catconfig = catconfig, cordconfig = cordconfig )
    
    oneat_model.X,oneat_model.X_val,oneat_model.Y,oneat_model.Y_val = create_train_val_data(img)
    
    oneat_model.Y = oneat_model.Y.reshape((oneat_model.Y.shape[0],1, 1, 1, oneat_model.Y.shape[1]))
    oneat_model.Y_val = oneat_model.Y_val.reshape((oneat_model.Y_val.shape[0],1, 1, 1, oneat_model.Y_val.shape[1]))
    oneat_model.TrainModel()
    keras_model = oneat_model.Trainingmodel
    assert keras_model.layers[1].output_shape[-1] == oneat_model.startfilter
    assert keras_model.layers[1].output_shape[1] is None
    assert keras_model.layers[1].output_shape[2] is None
    assert keras_model.layers[1].output_shape[3] is None
    
    assert keras_model.layers[-1].output_shape[-1] == len(catconfig) + len(cordconfig)
    assert keras_model.layers[-1].output_shape[1] is None
    assert keras_model.layers[-1].output_shape[2] is None
    assert keras_model.layers[-1].output_shape[3] is None
    
    assert oneat_model.yolo_loss.__name__ == 'loss'
    
    
